<?php
namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Notification;
use Teknisa\MAN\Utilities\Util;

use Teknisa\MAN\Services\ParaDeliveryService;

class ParaDelivery extends CrudWithParameterBag {
    
    protected $util;
    protected $parameterBag;
    protected $dataSourceNameArray = array (
        'filterModoDlv'            => 'paraDelivery_idmododlv',
        'filterCliente'            => 'paraDelivery_cdcliente',
        'filterFilial'             => 'paraDelivery_Filial',
        'filterVendedor'           => 'paraDelivery_Filial_cdvendedor',
        'filterBlockedCards'       => 'paraDelivery_BlockedCards',
        'filterLojaParamsDelivery' => 'paraDelivery_Loja_paramsDelivery',
        'filterLojaDias'           => 'paraDelivery_Loja_dias',
        'filterFilialAreaAtend'    => 'paraDelivery_Filial_AreaAtend',
        'filterAreaAtend'          => 'paraDelivery_AreaAtend',
        'filterCDAREA'             => 'paraDelivery_Filial_cdarea',
        'dataSourceRegraPromocional'=>'paraDelivery_regraPromocionalgeral',
        'dataSourceRegras'         => 'paraDelivery_selecionarRegraPromocional',
        'dataSourcePromoHour'      => 'paraDelivery_GeneralPromotionalHours'    
    );
    protected $parameterBagColumnsArray = array (
        'filterModoDlv'            =>  array('NRORG'),
        'filterCliente'            =>  array('NRORG', 'CDCLIENTE'),
        'filterFilial'             =>  array('NRORG', 'CDFILIAL'),
        'filterVendedor'           =>  array('NRORG', 'CDFILIAL'),
        'filterBlockedCards'       =>  array('NRORG'),
        'filterLojaParamsDelivery' =>  array('NRORG', 'CDFILIAL', 'CDLOJA'),
        'filterLojaDias'           =>  array('NRORG', 'CDFILIAL', 'CDLOJA', 'IDTPATENDELOJA'),
        'filterFilialAreaAtend'    =>  array('NRORG', 'CDFILIAL'),
        'filterAreaAtend'          =>  array('NRORG'),
        'filterCDAREA'             =>  array('NRORG', 'CDFILIAL'),
        'filterRegraPromocional'   =>  array('NRORG'),
        'filterPromoHour'          =>  array('NRORG'),
    );
    protected $paraDeliveryService;
    
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, ParaDeliveryService $paraDeliveryService, Util $util) {
        parent::__construct($dataSourceManager, $parameterBag);
        $this->paraDeliveryService = $paraDeliveryService;
        $this->util = $util;
    }
    
    public function findModoDlv(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterModoDlv'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['filterModoDlv'];
        parent::find($request, $response);
    }
    
    public function saveModoDlv(DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterModoDlv'];
        try {
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceName);
            $persistedRows = $this->dataSourceManager->persist($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceName, $persistedRows));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function findCliente(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterCliente'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['filterCliente'];
        parent::find($request, $response);
    }
    
    public function findFilial(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterFilial'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['filterFilial'];
        parent::find($request, $response);
    }
    
    public function saveFilial(DTO\Request\DataSet $request, DTO\Response $response) {
       $this->dataSourceName = $this->dataSourceNameArray['filterFilial'];
       parent::save($request,$response);
    }
    
    public function findVendedor(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterVendedor'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['filterVendedor'];
        parent::find($request, $response);
    }
    
    public function findBlockedCards(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterBlockedCards'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['filterBlockedCards'];
        parent::find($request, $response);
    }
    
    public function saveBlockedCards(DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterBlockedCards'];
        try {
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceName);
            $persistedRows = $this->dataSourceManager->persist($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceName, $persistedRows));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function removeBlockedCards(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $NRORG           = $row['NRORG'];
        $NRSEQCARTAOBLOQ = $row['NRSEQCARTAOBLOQ'];
        $this->paraDeliveryService->deleteBlockedCard($NRORG, $NRSEQCARTAOBLOQ);
    }
    
    public function findLojaParamsDelivery(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterLojaParamsDelivery'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['filterLojaParamsDelivery'];
        parent::find($request, $response);
    }
    
    public function saveLojaParamsDelivery(DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterLojaParamsDelivery'];
        try {
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceName);
            $persistedRows = $this->dataSourceManager->persist($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceName, $persistedRows));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function findLojaDias(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterLojaDias'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['filterLojaDias'];
        parent::find($request, $response);
    }
    
    public function saveLojaDias(DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterLojaDias'];
        $row = $request->getDataSet()->getRows();
        $NRORG           = $row[0]['NRORG'];
        $CDFILIAL        = $row[0]['CDFILIAL'];
        $CDLOJA          = $row[0]['CDLOJA']; 
        $NRDIAATENDELOJA = $row[0]['NRDIAATENDELOJA']; 
        $HRINIATENDELOJA = $row[0]['HRINIATENDELOJA']; 
        $HRFIMATENDELOJA = $row[0]['HRFIMATENDELOJA']; 
        $IDTPATENDELOJA  = $row[0]['IDTPATENDELOJA'];
        $NRSEQATENDELOJA = $row[0]['__is_new'] ? $this->util->newCode('ATENDELOJA', '100') : $row[0]['NRSEQATENDELOJA'];
        $verifica = $this->paraDeliveryService->verificaLojaDia($NRORG, $CDFILIAL, $CDLOJA, $NRDIAATENDELOJA, $HRINIATENDELOJA, $HRFIMATENDELOJA, $IDTPATENDELOJA, $NRSEQATENDELOJA);
        if (empty($verifica)) {
            if ($row[0]['__is_new']){
                $DTINCLUSAO     = $row[0]['DTINCLUSAO'];
                $DTULTATU       = $row[0]['DTULTATU'];
                $NRORGINCLUSAO  = $row[0]['NRORGINCLUSAO'];
                $CDOPERINCLUSAO = $row[0]['CDOPERINCLUSAO'];
                $NRORGULTATU    = $row[0]['NRORGULTATU'];
                $CDOPERULTATU   = $row[0]['CDOPERULTATU'];
                $IDATENDELOJA   = $row[0]['IDATENDELOJA'];
                $this->paraDeliveryService->insertLojaDia($NRORG, $CDFILIAL, $CDLOJA, $NRDIAATENDELOJA, $HRINIATENDELOJA, $HRFIMATENDELOJA, $IDTPATENDELOJA, $NRSEQATENDELOJA, $DTINCLUSAO, $DTULTATU, $NRORGINCLUSAO, $CDOPERULTATU, $CDOPERINCLUSAO, $NRORGULTATU, $IDATENDELOJA);
                
            } else
                try {
                    $dataSet = $request->getDataSet();
                    $dataSet->setDataSourceName($this->dataSourceName);
                    $persistedRows = $this->dataSourceManager->persist($dataSet);
                    $response->addDataSet(new DataSet($this->dataSourceName, $persistedRows));
                } catch (\Exception $e) {
                    $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
                }
        }        
        else
            $response->setError(new DTO\Response\Error('Intervalo de horário coincide com horário pré cadastrado.', 1));
    }
    
    public function removeLojaDias(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $NRORG           = $row['NRORG'];
        $CDFILIAL        = $row['CDFILIAL'];
        $CDLOJA          = $row['CDLOJA']; 
        $NRSEQATENDELOJA = $row['NRSEQATENDELOJA'];
        $this->paraDeliveryService->deleteLojaDia($NRORG, $CDFILIAL, $CDLOJA, $NRSEQATENDELOJA);
        
    }
    
    public function findFilialAreaAtend(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterFilialAreaAtend'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['filterFilialAreaAtend'];
        parent::find($request, $response);
    }
    
    public function saveFilialAreaAtend(DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterFilialAreaAtend'];
         try {
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceName);
            $persistedRows = $this->dataSourceManager->persist($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceName, $persistedRows));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function removeFilialAreaAtend(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $NRORG           = $row['NRORG'];
        $CDFILIAL        = $row['CDFILIAL'];
        $CDAREA          = $row['CDAREA']; 
        $this->paraDeliveryService->deleteFilialAreaAtend($NRORG, $CDFILIAL, $CDAREA);
        
    }
    
    public function findAreaAtend(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterAreaAtend'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['filterAreaAtend'];
        parent::find($request, $response);
    }    
    
    public function saveAreaAtendFile(DTO\Request\Row $request, DTO\Response $response){
		$row = $request->getRow();
        $coordinatesArray = $row['coordinates'];
        $fileName       = $row['fileName'];
        $row['CDAREA'] = $this->util->newCode('CDAREA', '10');
        $breakLine = array('\n', " ");
        $row['COORDAREA'] = str_replace($breakLine, "",json_encode($coordinatesArray));
        $row['NMAREA'] =  "ÁREA A SER EDITADA - ".$fileName;
        $row['DSAREA'] =  "ÁREA A SER EDITADA - ".$fileName;
        $row['HRINIATENDELOJA'] = null;
        $row['HRFIMATENDELOJA'] = null;
        
        if (strlen($row["NMAREA"]) > 50) {
            $row["NMAREA"] = substr($row["NMAREA"], 0, 50);
        }
        $this->paraDeliveryService->insertAreaAtend($row);
	}
    
    public function saveAreaAtend(DTO\Request\DataSet $request, DTO\Response $response) {
        $row = $request->getDataSet()->getRows();
        if (isset($row[1])){
            $row[0] = $row[1];
            array_pop($row);
        }
        $row = $row[0];
        
        if (!array_key_exists('HRINIATENDELOJA', $row)) {
            $row['HRINIATENDELOJA'] = null;
        }
        
        if (!array_key_exists('HRFIMATENDELOJA', $row)) {
            $row['HRFIMATENDELOJA'] = null;
        }
        
        if (is_array($row['COORDAREA'])) {
		    $row['COORDAREA'] = str_replace(array('\n', " "), "", json_encode($row['COORDAREA']));
        }
        
        if ($row['__is_new']){
            $row['CDAREA'] = $this->util->newCode('CDAREA', '10');
            $this->paraDeliveryService->insertAreaAtend($row);
        } else {
            $this->paraDeliveryService->updateAreaAtend($row);
            
        }
    }
    
    public function removeAreaAtend(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $NRORG           = $row['NRORG'];
        $CDAREA          = $row['CDAREA']; 
        $this->paraDeliveryService->deleteAreaAtend($NRORG, $CDAREA);
        
    }
    
    public function findCDAREA(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterCDAREA'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['filterCDAREA'];
        parent::find($request, $response);
    }
    
    public function findRegraPromocional(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['dataSourceRegraPromocional'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['filterRegraPromocional'];
        parent::find($request, $response);
    }
    
    public function saveRegraPromocional(DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['dataSourceRegraPromocional'];
        try {
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceName);
            $persistedRows = $this->dataSourceManager->persist($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceName, $persistedRows));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function findRegrasPromocionais(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['dataSourceRegras'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['filterRegraPromocional'];
        parent::find($request, $response);
    }
    
    public function findGeneralPromotionalHours(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['dataSourcePromoHour'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['filterPromoHour'];
        parent::find($request, $response);
    }
    
    public function saveHoraPromocao(DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['dataSourcePromoHour'];
        
        try {
            $row = $request->getDataSet()->getRows();
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceName);
            $persistedRows = $this->dataSourceManager->persist($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceName, $persistedRows));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function deleteHoraPromocao(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $NRORG           = $row['NRORG'];
        $NRDIAREGPROMO = $row['NRDIAREGPROM'];
        $this->paraDeliveryService->deleteHoraPromocao($NRORG, $NRDIAREGPROMO);
    }
    
    public function delete (DTO\Request\DataSet $request, DTO\Response $response){
        $this->dataSourceName = $this->dataSourceNameArray['filterLojaDias'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['filterLojaDias'];
        parent::delete($request,$response);
    }
    
     public function deleteMultiploAreaAtendimento ($request, $response){
        try {
            $dataSet = $request->getDataSet();
            $this->dataSourceName = $this->dataSourceNameArray['filterAreaAtend'];
            $dataSet->setDataSourceName($this->dataSourceName);
            $deletedRows = $this->dataSourceManager->delete($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceName, $deletedRows));
            $nrDeletedRows = count($deletedRows);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
}  